@@ -210,6 +210,9 @@ struct
   int autoimport;            /* Automatically import keys from headers or attachments. */
   char *draft_key;           /* Encrypt drafts with this key (fingerprint) */
   int alwaysShowApproval;    /* Always show the certificate approval dialog. */
+  int combinedOpsEnabled;    /* Enable S/MIME and OpenPGP combined operations. */
+  int splitBCCMails;         /* Split BCC recipients in their own mails. */
+  int encryptSubject;        /* Encrypt the subject with protected headers. */
 
   /* The forms revision number of the binary.  */
   int forms_revision;
@@ -335,6 +338,14 @@ struct autocrypt_s
   std::string addr;
   GpgME::Data data;
 };
+
+/* A general way to transfer some header infos around
+   without having to parse them multiple times. */
+struct header_info_s
+{
+  std::string boundary;
+  autocrypt_s acInfo;
+};
 #endif
 
 #endif // COMMON_INDEP_H
