@@ -29,11 +29,11 @@
 #include "gpgme++/key.h"
 
 #include <string>
-#include <future>
 
 class ParseController;
 class CryptController;
 class Attachment;
+class Recipient;
 
 /** @brief Data wrapper around a mailitem.
  *
@@ -340,6 +340,10 @@ public:
     */
   bool isSMIME_m ();
 
+  /* Same as above but does not touch MAPI. isSMIME_m needs to
+     be called before this can return a true value. */
+  bool isSMIME () const;
+
   /** @brief get the associated parser.
     only valid while the actual parsing happens. */
   std::shared_ptr<ParseController> parser () { return m_parser; }
@@ -352,8 +356,11 @@ public:
    In Qt this would be a slot that is called once it is finished
    we hack around that a bit by calling it from our windowmessages
    handler.
+
+   Parsing done can be called twice, once for the preview of
+   a signed message and once when everything is finished.
   */
-  void parsing_done ();
+  void parsingDone_o (bool is_preview = false);
 
   /** Returns true if the mail was verified and has at least one
     signature. Regardless of the validity of the mail */
@@ -400,9 +407,6 @@ public:
   /** Get the body of the mail */
   std::string getBody_o () const;
 
-  /** Get the recipients. */
-  std::vector<std::string> getRecipients_o () const;
-
   /** Try to locate the keys for all recipients.
       This also triggers the Addressbook integration, which we
       treat as locate jobs. */
@@ -427,7 +431,16 @@ public:
   char *takeCachedPlainBody ();
 
   /** Get the cached recipients. It is updated in update_oom_data.*/
-  std::vector<std::string> getCachedRecipients ();
+  std::vector<Recipient> getCachedRecipients ();
+  /** Only get the Mail addresses from the recipient objects. */
+  std::vector<std::string> getCachedRecipientAddresses ();
+
+  /** Set an override for recipients. If they have keys the
+   keys will not be resolved again. */
+  void setRecipients (const std::vector<Recipient> &recps);
+
+  /** Get the recipients. */
+  std::vector<Recipient> getRecipients_o () const;
 
   /** Returns 1 if the mail was encrypted, 2 if signed, 3 if both.
       Only valid after decrypt_verify.
@@ -514,9 +527,9 @@ public:
   */
   HWND getWindow () { return m_window; }
 
-  /** Cleanup any attached crypter object. Useful
+  /** Cleanup any attached crypter objects. Useful
     on error. */
-  void resetCrypter () { m_crypter = nullptr; }
+  void resetCrypter ();
 
   /** Set special crypto mime data that should be used as the
     mime structure when sending. */
@@ -530,10 +543,6 @@ public:
   void setIsForwardedCryptoMail (bool value) { m_is_forwarded_crypto_mail = value; }
   bool is_forwarded_crypto_mail () { return m_is_forwarded_crypto_mail; }
 
-  /** Set if this is a reply of a crypto mail. */
-  void setIsReplyCryptoMail (bool value) { m_is_reply_crypto_mail = value; }
-  bool is_reply_crypto_mail () { return m_is_reply_crypto_mail; }
-
   /** Remove the hidden GpgOL attachments. This is needed when forwarding
     without encryption so that our attachments are not included in the forward.
     Returns 0 on success. Works in OOM. */
@@ -556,7 +565,10 @@ public:
     */
   bool hasCryptedOrEmptyBody_o ();
 
-  void updateBody_o ();
+  void updateBody_o (bool is_preview);
+
+  /** Update information from protected headers in OOM */
+  void updateHeaders_o ();
 
   /** Set if this mail looks like the send again of a crypto mail.
       This will mean that after it is decrypted it is treated
@@ -622,7 +634,7 @@ public:
   void decryptPermanently_o ();
 
   /* Prepare for encrypt / sign. Updates data. */
-  void prepareCrypto_o ();
+  int prepareCrypto_o ();
 
   /* State variable to check if we are about to encrypt a draft. */
   void setIsDraftEncrypt (bool value) { m_is_draft_encrypt = value; }
@@ -649,10 +661,55 @@ public:
      crypt results. */
   void updateCategories_o ();
 
+  /* Set the signing key for the mail. This overrides any other
+     key resolution. */
+  void setSigningKeys (const std::vector<GpgME::Key> &keys);
+  std::vector <GpgME::Key> getSigningKeys () const;
+
+  /* Set the recipients for this mail. If they all have keys
+  set then this overrides the cryptcontroller resolutions*/
+  void setRecipients (const std::vector<GpgME::Key> &keys);
+
+  /* Split a mail according to recipients, hidden, different protocols
+     and so on and then send the multiple mails. */
+  void splitAndSend_o ();
+
+  /* A callback for another mail that was created as
+     a split from us. */
+  void splitCopyMailCallback (Mail *copy);
+
+  /* Reset recipient and resolved key data. */
+  void resetRecipients ();
+
+  /* Returns true if a mail is a copy that was split of
+     a different mail. */
+  bool isSplitCopy () const;
+
+  /* Setter for isSplitCopy */
+  void setSplitCopy (bool val);
+
+  /* Set protected headers data */
+  void setProtectedHeaders (const std::string &hdrs);
+  std::string protectedHeaders () const;
+
+  /* Parse the headers so that the header_info_s
+     becomes valid.*/
+  int parseHeaders_m ();
+  /* Get the header info struct. Only valid
+     afer parseHeaders_m */
+  header_info_s headerInfo () const;
+
+  /* Get the original body of the mail. */
+  const std::string &getOriginalBody () const;
 private:
-  bool checkIfMailIsChildOfPrintMail_o ();
+  /* Returns a copy of the mailitem object. This copy
+     is sadly not the same as in the ItemLoad event
+     so we cannot find it in our map of mailitems. */
+  LPDISPATCH copy ();
+  bool checkIfMailMightBePrinting_o ();
   void updateSigstate ();
   int add_attachments_o (std::vector<std::shared_ptr<Attachment> > attachments);
+  int buildProtectedHeaders_o ();
 
   LPDISPATCH m_mailitem;
   LPDISPATCH m_event_sink;
@@ -674,7 +731,7 @@ private:
   std::string m_sent_on_behalf;
   char *m_cached_html_body; /* Cached html body. */
   char *m_cached_plain_body; /* Cached plain body. */
-  std::vector<std::string> m_cached_recipients;
+  std::vector<Recipient> m_cached_recipients;
   msgtype_t m_type; /* Our messagetype as set in mapi */
   std::shared_ptr <ParseController> m_parser;
   std::shared_ptr <CryptController> m_crypter;
@@ -709,5 +766,16 @@ private:
                            another beforeread */
   bool m_printing; /* Mail is decrypted for printing */
   std::string m_gpgol_class; /* The GpgOL Message class */
+  std::vector<GpgME::Key> m_resolved_signing_keys; /* Prepared / resolved keys for signing. */
+  bool m_recipients_set; /* Recipients were explictly set. */
+  bool m_is_split_copy; /* Is the a copy mail that was part of a split. */
+  std::string m_protected_headers;
+  header_info_s m_header_info; /* Information about the original headers */
+  bool m_attachs_added; /* State variable to track if we have added attachments to this mail. */
+  std::string m_dec_content_type; /* Top level content type of the decrypted mail. */
 };
+
+/* A state variable to capture which mail triggered a copy to
+   use our own objects in the ItemLoad event. */
+extern Mail *g_mail_copy_triggerer;
 #endif // MAIL_H
