@@ -4,17 +4,17 @@
  * Software engineering by Intevation GmbH
  *
  * This file is part of GpgOL.
- * 
+ *
  * GpgOL is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * GpgOL is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
@@ -36,7 +36,9 @@
 #define MSOCONTROLCOMBOBOX  4
 #define MSOCONTROLPOPUP    10
 
-enum 
+class Recipient;
+
+enum
   {
     msoButtonAutomatic = 0,
     msoButtonIcon = 1,
@@ -45,7 +47,7 @@ enum
     msoButtonIconAndWrapCaption = 7,
     msoButtonIconAndCaptionBelow = 11,
     msoButtonWrapCaption = 14,
-    msoButtonIconAndWrapCaptionBelow = 15 
+    msoButtonIconAndWrapCaptionBelow = 15
   };
 
 enum
@@ -58,10 +60,10 @@ enum
 
 DEFINE_GUID(GUID_NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
 
-DEFINE_GUID(IID_IConnectionPoint, 
+DEFINE_GUID(IID_IConnectionPoint,
             0xb196b286, 0xbab4, 0x101a,
             0xb6, 0x9c, 0x00, 0xaa, 0x00, 0x34, 0x1d, 0x07);
-DEFINE_GUID(IID_IConnectionPointContainer, 
+DEFINE_GUID(IID_IConnectionPointContainer,
             0xb196b284, 0xbab4, 0x101a,
             0xb6, 0x9c, 0x00, 0xaa, 0x00, 0x34, 0x1d, 0x07);
 DEFINE_GUID(IID_IPictureDisp,
@@ -150,8 +152,13 @@ DEFINE_OLEGUID(IID_IOleWindow,                0x00000114, 0, 0);
 #define PR_SENT_REPRESENTING_NAME_W_DASL \
   "http://schemas.microsoft.com/mapi/proptag/0x0042001F"
 
+#define PR_SECURITY_FLAGS_DASL \
+  "http://schemas.microsoft.com/mapi/proptag/0x6E010003"
+
 #define DISTRIBUTION_LIST_ADDRESS_ENTRY_TYPE 11
 
+#define DISPID_DIRTY_RAT 0xF024
+
 typedef std::shared_ptr<IDispatch> shared_disp_t;
 
 /* Function to contain the gpgol_release macro */
@@ -161,12 +168,17 @@ void release_disp (LPDISPATCH obj);
 
 /* Return the malloced name of an COM+ object.  */
 char *get_object_name (LPUNKNOWN obj);
+std::string get_object_name_s (LPUNKNOWN obj);
+std::string get_object_name_s (shared_disp_t obj);
 
 /* Helper to lookup a dispid.  */
 DISPID lookup_oom_dispid (LPDISPATCH pDisp, const char *name);
 
 /* Return the OOM object's IDispatch interface described by FULLNAME.  */
 LPDISPATCH get_oom_object (LPDISPATCH pStart, const char *fullname);
+/* Do the same but with a shared disp return value */
+shared_disp_t get_oom_object_s (LPDISPATCH pStart, const char *fullname);
+shared_disp_t get_oom_object_s (shared_disp_t pStart, const char *fullname);
 
 /* Set the Icon of a CommandBarControl.  */
 int put_oom_icon (LPDISPATCH pDisp, int rsource_id, int size);
@@ -183,14 +195,21 @@ int put_oom_string (LPDISPATCH pDisp, const char *name, const char *string);
 /* Set the property NAME to DISP.  */
 int put_oom_disp (LPDISPATCH pDisp, const char *name, LPDISPATCH value);
 
+/* Set the byte array property NAME to VALUE. */
+int put_oom_array (LPDISPATCH pDisp, const char *name,
+                   unsigned char *value, size_t size);
+
 /* Get the boolean property NAME of the object PDISP.  */
 int get_oom_bool (LPDISPATCH pDisp, const char *name);
 
 /* Get the integer property NAME of the object PDISP.  */
 int get_oom_int (LPDISPATCH pDisp, const char *name);
+int get_oom_int (shared_disp_t pDisp, const char *name);
 
 /* Get the string property NAME of the object PDISP.  */
 char *get_oom_string (LPDISPATCH pDisp, const char *name);
+std::string get_oom_string_s (LPDISPATCH pDisp, const char *name);
+std::string get_oom_string_s (shared_disp_t pDisp, const char *name);
 
 /* Get an IUnknown object from property NAME of PDISP.  */
 LPUNKNOWN get_oom_iunknown (LPDISPATCH pDisp, const char *name);
@@ -209,13 +228,15 @@ void del_oom_button (LPDISPATCH button);
 HWND get_oom_context_window (LPDISPATCH context);
 
 /* Get the address of the recipients as string list.
+   The second part of the pair returned is the recipient type
+   which corresponds in value to Mail::recipientType.
    If r_err is not null it is set to true in case of an error. */
-std::vector<std::string> get_oom_recipients (LPDISPATCH recipients,
-                                             bool *r_err = nullptr);
+std::vector<Recipient> get_oom_recipients (LPDISPATCH recipients,
+                                           bool *r_err = nullptr);
 
 /* Same as above but include the AddrEntry object in the result.
    Caller needs to release the AddrEntry. */
-std::vector<std::pair<std::string, shared_disp_t> >
+std::vector<std::pair<Recipient, shared_disp_t> >
 get_oom_recipients_with_addrEntry (LPDISPATCH recipients,
                                    bool *r_err = nullptr);
 
@@ -331,6 +352,16 @@ int
 invoke_oom_method_with_parms (LPDISPATCH pDisp, const char *name,
                               VARIANT *rVariant, DISPPARAMS *params);
 
+/* Same as invoke oom method but do the string marshalling for arg */
+int invoke_oom_method_with_string (LPDISPATCH pDisp, const char *name,
+                                   const char *arg,
+                                   VARIANT *rVariant = nullptr);
+
+/* Same as invoke oom method but with a single int argument */
+int invoke_oom_method_with_int (LPDISPATCH pDisp, const char *name,
+                                int arg,
+                                VARIANT *rVariant = nullptr);
+
 /* Try to obtain the mapisession through the Application.
   returns NULL on error.*/
 LPMAPISESSION
@@ -443,4 +474,26 @@ std::string format_dispparams (DISPPARAMS *p);
 
 /* Returns the count of attachments that are not hidden. */
 int count_visible_attachments (LPDISPATCH attachments);
+
+/* Remove a recipient from the OOM. */
+int remove_oom_recipient (LPDISPATCH item, const std::string &mbox);
+
+/* Remove all recipients and replace them with the list of our objects. */
+int set_oom_recipients (LPDISPATCH item, const std::vector<Recipient> &recps);
+
+/* Print some introspective infos about the object. */
+void oom_dump_idispatch (LPDISPATCH obj);
+
+/* Get the hidden dirty property of the object. */
+int get_oom_dirty (LPDISPATCH pDisp);
+
+/* Get the Outlook crypto flags indicating in Outlook if the mail should
+   be encrypted with S/MIME. */
+int get_oom_crypto_flags (LPDISPATCH mailitem);
+
+/* Setter for dirty - Returns an error that the property
+   is write protected on mails. But maybe we can use it
+   somewhere else. Code is ifdeffed out.
+int put_oom_dirty (LPDISPATCH pDisp, bool val);
+ */
 #endif /*OOMHELP_H*/
