@@ -114,7 +114,9 @@ ParseController::ParseController(LPSTREAM instream, msgtype_t type):
                           expect_no_headers(type))),
     m_outputprovider (new MimeDataProvider(expect_no_mime(type))),
     m_type (type),
-    m_block_html (false)
+    m_block_html (false),
+    m_second_pass (false)
+
 {
   TSTART;
   memdbg_ctor ("ParseController");
@@ -131,7 +133,8 @@ ParseController::ParseController(FILE *instream, msgtype_t type):
                           expect_no_headers(type))),
     m_outputprovider (new MimeDataProvider(expect_no_mime(type))),
     m_type (type),
-    m_block_html (false)
+    m_block_html (false),
+    m_second_pass (false)
 {
   TSTART;
   memdbg_ctor ("ParseController");
@@ -331,7 +334,6 @@ is_valid_chksum(const GpgME::Signature &sig)
   TRETURN valid;
 }
 
-
 /* Note on stability:
 
    Experiments have shown that we can have a crash if parse
@@ -345,7 +347,7 @@ is_valid_chksum(const GpgME::Signature &sig)
    (slower e.g. when pinentry is requrired).
 */
 void
-ParseController::parse()
+ParseController::parse(bool offline)
 {
   TSTART;
   // Wrap the input stream in an attachment / GpgME Data
@@ -375,6 +377,12 @@ ParseController::parse()
       is_smime (input))
     {
       protocol = Protocol::CMS;
+      if (m_second_pass)
+        {
+          log_dbg ("Second pass parsing for CMS. Only doing verify.");
+          verify = true;
+          decrypt = false;
+        }
     }
   else
     {
@@ -417,6 +425,10 @@ ParseController::parse()
     {
       ctx->setSender(m_sender.c_str());
     }
+  if (offline)
+    {
+      ctx->setOffline (true);
+    }
 
   Data output (m_outputprovider);
   log_debug ("%s:%s:%p decrypt: %i verify: %i with protocol: %s sender: %s type: %i",
@@ -556,6 +568,8 @@ ParseController::parse()
   log_debug ("%s:%s:%p: decrypt err: %i verify err: %i",
              SRCNAME, __func__, this, m_decrypt_result.error().code(),
              m_verify_result.error().code());
+  /* If we are called again it is the second pass */
+  m_second_pass = true;
 
   bool has_valid_encrypted_checksum = false;
   /* Ensure that the Keys for the signatures are available
@@ -566,7 +580,11 @@ ParseController::parse()
       has_valid_encrypted_checksum = is_valid_chksum (sig);
 
 #ifndef BUILD_TESTS
-      KeyCache::instance ()->update (sig.fingerprint (), protocol);
+      /* For TOFU we would need to update here even when offline. */
+      if (!offline || KeyCache::instance ()->useTofu ())
+        {
+          KeyCache::instance ()->update (sig.fingerprint (), protocol);
+        }
 #endif
       TRACEPOINT;
     }
@@ -704,12 +722,26 @@ ParseController::get_attachments() const
 }
 
 std::string
-ParseController::get_internal_subject() const
+ParseController::get_protected_header (const std::string &which) const
 {
   TSTART;
   if (m_outputprovider)
     {
-      TRETURN m_outputprovider->get_internal_subject ();
+      TRETURN m_outputprovider->get_protected_header (which);
+    }
+  TRETURN std::string ();
+}
+
+std::string
+ParseController::get_content_type () const
+{
+  TSTART;
+  if (m_outputprovider)
+    {
+      TRETURN m_outputprovider->get_content_type ();
+    }
+  else
+    {
+      TRETURN std::string();
     }
-  TRETURN std::string();
 }
