@@ -772,30 +772,38 @@ _nl_locale_name (int category, const char *categoryname)
      context, because message catalogs are not specific to a single
      codeset.  */
 
-  LCID lcid;
+  static LCID lcid = 0;
   LANGID langid;
   int primary, sub;
 
   (void)category;
 
+  if (!lcid)
+    {
+      /* We only check once because Outlook also requires
+         a restart after switching the language. This fixes
+         a problem that in some callbacks
+         Application.LanguageSettings is not available.*/
+        /* Prefer the Ui language of Outlook. */
+#ifndef BUILD_TESTS
+        lcid = get_ol_ui_language ();
+#endif
+    }
+
   /* Let the user override the system settings through environment
      variables, as on POSIX systems.  */
-  retval = getenv ("LC_ALL");
-  if (retval != NULL && retval[0] != '\0')
-    return retval;
-  retval = getenv (categoryname);
-  if (retval != NULL && retval[0] != '\0')
-    return retval;
-  retval = getenv ("LANG");
-  if (retval != NULL && retval[0] != '\0')
-    return retval;
-
-  /* Prefer the Ui language of Outlook. */
-#ifndef BUILD_TESTS
-  lcid = get_ol_ui_language ();
-#else
-  lcid = 0;
-#endif
+  if (!lcid)
+    {
+      retval = getenv ("LC_ALL");
+      if (retval != NULL && retval[0] != '\0')
+        return retval;
+      retval = getenv (categoryname);
+      if (retval != NULL && retval[0] != '\0')
+        return retval;
+      retval = getenv ("LANG");
+      if (retval != NULL && retval[0] != '\0')
+        return retval;
+    }
 
   if (!lcid)
     {
